require 'amsi/validator/request_errors'

require_relative 'base'

module Amsi
  module DocumentParser
    # Parse the GetPropertyList response
    class Properties < Base
      ADDRESS_ATTRS = %i[
        Line1
        Line2
        Line3
        Line4
        City
        State
        ZipCode
        Country
      ].freeze
      private_constant :ADDRESS_ATTRS

      MANAGER_ATTR_MAP = {
        'MgrOffPhoneNo' => 'phone',
        'MgrFaxNo' => 'fax',
        'MgrSalutation' => 'salutation',
        'MgrFirstName' => 'first_name',
        'MgrMiName' => 'middle_name',
        'MgrLastName' => 'last_name'
      }.freeze
      private_constant :MANAGER_ATTR_MAP

      private

      # @param body [Hash<String, Object>] the body of the XML response parsed
      #   into a Hash
      # @return [Array<Amsi::Model::Property>] the properties contained in
      #   the response
      # @raise [Amsi::Error::Base] if the response is invalid
      def parse_body(body)
        property_hashes(body).map do |property_hash|
          property = Model::Property.new(property_hash)
          property.address = address(property_hash, 'Property')
          property.remit_to_address = address(property_hash, 'RemitTo')
          property.manager = manager(property_hash)
          property.leasing_agents = leasing_agents(property_hash)
          property.marketing_sources = marketing_sources(property_hash)
          property.unit_types = unit_types(property_hash)
          property
        end
      end

      def address(property_hash, prefix)
        address_attrs = ADDRESS_ATTRS.map.with_object({}) do |field, hash|
          hash[field] = property_hash["#{prefix}Addr#{field}"]
        end
        Model::Address.new(address_attrs)
      end

      def manager(property_hash)
        manager_attrs = MANAGER_ATTR_MAP.map.with_object({}) do |(node, attr), hash|
          hash[attr] = property_hash[node]
        end
        Model::Manager.new(manager_attrs)
      end

      def leasing_agents(property_hash)
        agent_hashes = property_hash['LeasingAgent']
        return [] if agent_hashes.nil?

        agent_hashes = [agent_hashes] unless agent_hashes.is_a?(Array)
        agent_hashes.map do |agent_hash|
          Model::LeasingAgent.new(agent_hash)
        end
      end

      def marketing_sources(property_hash)
        source_hashes = property_hash['MarketingSource']
        return [] if source_hashes.nil?

        source_hashes = [source_hashes] unless source_hashes.is_a?(Array)
        source_hashes.map do |source_hash|
          Model::MarketingSource.new(source_hash)
        end
      end

      def unit_types(property_hash)
        type_hashes = property_hash['UnitType']
        return [] if type_hashes.nil?

        type_hashes = [type_hashes] unless type_hashes.is_a?(Array)
        type_hashes.map do |type_hash|
          Model::UnitType.new(type_hash)
        end
      end

      def property_hashes(body)
        response = body['GetPropertyListResponse']
        escaped_result = response['GetPropertyListResult']
        parsed_result = MultiXml.parse(escaped_result)
        properties = parsed_result['Properties']['Property']

        return [] if properties.nil?

        properties.is_a?(Array) ? properties : [properties]
      end
    end
  end
end
